/*
 * This file is part of Sounds.
 *
 *   Sounds is free software: you can redistribute it and/or modify
 *   it under the terms of the Lesser GNU General Public License
 *   version 3 (LGPL3) as published by the Free Software Foundation.
 *
 *   Sounds is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the Lesser GNU General Public License
 *   along with Sounds.  If not, see <https://www.gnu.org/licenses/>.
 */
import 'dart:async';

import 'package:path/path.dart';

import 'ffmpeg/ffmpeg_util.dart';

/// Describes the set of possible codecs.
/// Not all codecs are support on all OSs nor all actions.
enum Codec {
  // this enum MUST be synchronized with sounds/AudioInterface.java
  // and ios/Classes/SoundsPlugin.h

  /// This is the default codec. If used
  /// Sounds will use the files extension to guess the codec.
  /// If the file extension doesn't match a known codec then
  /// Sounds will throw an exception in which case you need
  /// pass one of the know codec.
  fromExtension,

  /// AAC codec in an ADTS container
  aacADTS,

  /// OPUS in an OGG container
  opusOGG,

  /// Apple encapsulates its bits in its own special envelope
  /// .caf instead of a regular ogg/opus (.opus).
  /// This is completely stupid, this is Apple.
  cafOpus,

  /// For those who really insist about supporting MP3. Shame on you !
  mp3,

  /// VORBIS in a OGG container
  vorbisOGG,

  /// Linear 16 PCM, which is a Wav file.
  pcm,

  /// AAC codec in a MPEG4 container
  aacLC,

  /// AAC codec in a 3GP container
  aac3GP,

  /// AAC in a MPEG4 container
  aacMP4,

  /// PCM in a .wav container
  wav,

  ///
  flac,
}

/// Helper functions for the Codec enum becuase
/// dart's enums are crap.
class CodecHelper {
  /// Provides mappings from common file extensions to
  /// the codec those files use.
  static const extensionToCodecMap = <String, Codec>{
    '.aac': Codec.aacADTS,
    '.opus': Codec.opusOGG,
    '.caf': Codec.cafOpus,
    '.mp3': Codec.mp3,
    '.ogg': Codec.vorbisOGG,
    '.wav': Codec.pcm,
    '.flac': Codec.flac,
  };

  /// Maps codecs to common file extensions.
  static const codecToExtensionMap = <Codec, String>{
    Codec.aacADTS: '.aac',
    Codec.opusOGG: '.opus',
    Codec.cafOpus: '.caf',
    Codec.mp3: '.mp3',
    Codec.vorbisOGG: '.ogg',
    Codec.pcm: '.wav',
    Codec.aacLC: '.aac',
    Codec.aac3GP: '.aac',
    Codec.aacMP4: '.aac',
    Codec.wav: '.wav',
    Codec.flac: '.flac',
  };

  /// Returns the codec for the given [path]
  /// by using the filename's extension.
  /// If the filename's extension doesn't match one of the supported
  /// file extensions listed in [extensionToCodecMap] then
  /// null is returned.
  static Codec determineCodec(String path) {
    var ext = extension(path);

    var codec = extensionToCodecMap[ext];

    return codec;
  }

  /// Determines the duration of the audio file located at [path]
  /// based on the files codec.
  static Future<Duration> duration(Codec codec, String path) async {
    var duration = Future<Duration>.value(Duration.zero);

    /// supports a large range of file formats.
    duration = FFMpegUtil().duration(path);

    return duration;
  }
}

/// Throw if an unsupported codec is passed.
/// Different OS's support a different set of codecs.
/// This can also be thrown if you pass in fromExtension
/// and the file extension isn't recognized.
class CodecNotSupportedException implements Exception {
  final String _message;

  ///
  CodecNotSupportedException(this._message);

  String toString() => _message;
}
